/**
 ****************************************************************************************************
 * @file        demo.c
 * @author      ԭŶ(ALIENTEK)
 * @version     V1.0
 * @date        2022-06-21
 * @brief       ATK-MO395QģUDPͨѶʵ
 * @license     Copyright (c) 2020-2032, ӿƼ޹˾
 ****************************************************************************************************
 * @attention
 *
 * ʵƽ̨:ԭ STM32F103
 * Ƶ:www.yuanzige.com
 * ̳:www.openedv.com
 * ˾ַ:www.alientek.com
 * ַ:openedv.taobao.com
 *
 ****************************************************************************************************
 */

#include "demo.h"
#include "./BSP/ATK_MO395Q/atk_mo395q.h"
#include "./BSP/LED/led.h"
#include "./BSP/KEY/key.h"
#include "./SYSTEM/usart/usart.h"
#include "./SYSTEM/delay/delay.h"
#include <string.h>

#define DEMO_SOCKET_PROTO       ATK_MO395Q_SOCKET_UDP   /* UDP */
#define DEMO_SOCKET_DES_IP_1    192                     /* ָUDPͻ˵IPַ */
#define DEMO_SOCKET_DES_IP_2    168
#define DEMO_SOCKET_DES_IP_3    1
#define DEMO_SOCKET_DES_IP_4    13
#define DEMO_SOCKET_DES_PORT    8080                    /* ָUDPͻ˵Ķ˿ں */
#define DEMO_SOCKET_SOUR_PORT   8080                    /* ض˿ں */

static uint8_t socket0_send_buf[] = {"This is from ATK-MO395Q\r\n"};
static uint8_t socket0_recv_buf[1024];
static uint8_t socket0_send_done = 1;

/**
 * @brief       PHYӻص
 * @param       phy_status: PHY״̬
 * @retval      
 */
static void demo_phy_conn_cb(uint8_t phy_status)
{
    printf("PHY Connected! ");
    
    switch (phy_status)
    {
        case ATK_MO395Q_CMD_PHY_10M_FLL:
        {
            printf("10Mbps Full\r\n");
            break;
        }
        case ATK_MO395Q_CMD_PHY_10M_HALF:
        {
            printf("10Mbps Half\r\n");
            break;
        }
        case ATK_MO395Q_CMD_PHY_100M_FLL:
        {
            printf("100Mbps Full\r\n");
            break;
        }
        case ATK_MO395Q_CMD_PHY_100M_HALF:
        {
            printf("100Mbps Half\r\n");
            break;
        }
        case ATK_MO395Q_CMD_PHY_AUTO:
        {
            printf("Auto\r\n");
            break;
        }
        default:
        {
            break;
        }
    }
    
    printf("\r\n");
}

/**
 * @brief       PHYϿص
 * @param       
 * @retval      
 */
static void demo_phy_disconn_cb(void)
{
    printf("PHY Disconnected!\r\n");
    printf("\r\n");
}

/**
 * @brief       DHCPɹص
 * @param       ip  : IPַ
 *              gwip: IPַ
 *              mask: 
 *              dns1: DNS1ַ
 *              dns2: DNS2ַ
 * @retval      
 */
static void demo_dhcp_success_cb(uint8_t *ip, uint8_t *gwip, uint8_t *mask, uint8_t *dns1, uint8_t *dns2)
{
    printf("IP: %d.%d.%d.%d\r\n", ip[0], ip[1], ip[2], ip[3]);
    printf("Gateway: %d.%d.%d.%d\r\n", gwip[0], gwip[1], gwip[2], gwip[3]);
    printf("Mask: %d.%d.%d.%d\r\n", mask[0], mask[1], mask[2], mask[3]);
    printf("DNS1: %d.%d.%d.%d\r\n", dns1[0], dns1[1], dns1[2], dns1[3]);
    printf("DNS2: %d.%d.%d.%d\r\n", dns2[0], dns2[1], dns2[2], dns2[3]);
    printf("\r\n");
}

/**
 * @brief       Socketͻлص
 * @param       socket: Socket
 * @retval      
 */
static void demo_socket_send_buf_free_cb(atk_mo395q_socket_t *socket)
{
    switch (socket->socket_index)
    {
        case ATK_MO395Q_SOCKET_0:
        {
            socket0_send_done = 1;
            break;
        }
        default:
        {
            break;
        }
    }
}

/**
 * @brief       Socketջǿջص
 * @param       socket: Socket
 * @retval      
 */
static void demp_socket_recv_cb(atk_mo395q_socket_t *socket)
{
    uint16_t recv_len;
    
    recv_len = atk_mo395q_cmd_get_recv_len_sn(socket->socket_index);
    if (recv_len != 0)
    {
        if (recv_len > (socket->recv.size - 1))
        {
            recv_len = socket->recv.size;
        }
        atk_mo395q_cmd_read_recv_buf_sn(socket->socket_index, recv_len, socket->recv.buf);
        socket->recv.buf[recv_len] = '\0';
        printf("%s", socket->recv.buf);
    }
}

/**
 * @brief       ʾں
 * @param       
 * @retval      
 */
void demo_run(void)
{
    uint8_t ret;
    uint8_t key;
    atk_mo395q_socket_t socket_config = {0};
    
    /* ʼATK-MO395Qģ */
    ret = atk_mo395q_init();
    if (ret != 0)
    {
        printf("ATK-MO395Q init failed!\r\n");
        while (1)
        {
            LED0_TOGGLE();
            delay_ms(200);
        }
    }
    
    /* ATK-MO395Qģ */
    atk_mo395q_net_config(ATK_MO395Q_ENABLE, NULL, NULL, NULL, demo_phy_conn_cb, demo_phy_disconn_cb, demo_dhcp_success_cb);
    
    /* ATK-MO395QģSocket */
    socket_config.socket_index = ATK_MO395Q_SOCKET_0;
    socket_config.enable = ATK_MO395Q_ENABLE;
    socket_config.proto = DEMO_SOCKET_PROTO;
    socket_config.des_ip[0] = DEMO_SOCKET_DES_IP_1;
    socket_config.des_ip[1] = DEMO_SOCKET_DES_IP_2;
    socket_config.des_ip[2] = DEMO_SOCKET_DES_IP_3;
    socket_config.des_ip[3] = DEMO_SOCKET_DES_IP_4;
    socket_config.des_port = DEMO_SOCKET_DES_PORT;
    socket_config.sour_port = DEMO_SOCKET_SOUR_PORT;
    socket_config.send.buf = socket0_send_buf;
    socket_config.send.size = sizeof(socket0_send_buf);
    socket_config.recv.buf =  socket0_recv_buf;
    socket_config.recv.size = sizeof(socket0_recv_buf);
    socket_config.send_buf_free_cb = demo_socket_send_buf_free_cb;
    socket_config.recv_cb = demp_socket_recv_cb;
    atk_mo395q_socket_config(&socket_config);
    
    while (1)
    {
        key = key_scan(0);
        atk_mo395q_handler();
        
        if ((key == KEY0_PRES) && (socket0_send_done == 1))
        {
            socket0_send_done = 0;
            atk_mo395q_cmd_write_send_buf_sn(ATK_MO395Q_SOCKET_0, socket0_send_buf, strlen((char *)socket0_send_buf));
        }
    }
}
